package com.ibm.edp.cic.core.ant;

import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.osgi.service.resolver.VersionRange;
import org.osgi.framework.Version;

import com.ibm.cic.ant.jar.OfferingId;
import com.ibm.cic.common.core.model.IIdentity;
import com.ibm.cic.common.core.model.IOffering;
import com.ibm.cic.common.core.repository.IRepository;
import com.ibm.edp.cic.core.helper.CicHelper;
import com.ibm.edp.cic.core.helper.Util;

public abstract class AbstractCicTransformOfferingMetadata extends AbstractCicTaskWrapper 
{	
	public static final Version versionUnspecified = new Version(0, 0, 0);
	
	private boolean commit = false;

    public void setCommit(boolean b)
    {
    	this.commit = b;
    }

    public boolean getCommit()
    {
    	return this.commit;
    }
    
    protected void doExecute() throws BuildException {

        if ( getDestMetadataDir() == null)
            log("destMetadataDir:null");//$NON-NLS-1$
        else
            log("destMetadataDir:" + getDestMetadataDir().toString());//$NON-NLS-1$
        
        dumpIds("Offerings", getOfferings() );//$NON-NLS-1$
        
        createRepositoryGroup("CicTransformOfferingMetadata"); //$NON-NLS-1$
        
        try {
        	
        	openSourceRepositories();
        	
        	IProgressMonitor monitor =  new NullProgressMonitor();
        	
            IRepository targetRepo = Util.obtainRepository( getDestMetadataDir().getAbsolutePath() );
            
            OfferingId offering = getFirstOffering();

			if(offering == null)
			{
                List offerings = targetRepo.getAllOfferings( monitor );
                for (Iterator iter = offerings.iterator(); iter.hasNext();) 
                {
					IOffering offer = (IOffering) iter.next();
					log("Offering: [" + offer.getIdentity().getId() + ", " + offer.getVersion() + "]");//$NON-NLS-1$
					List logs = transform(offer, targetRepo, monitor);
					print( logs );
				}
			}
			else
			{
				IIdentity offeringId = CicHelper.getIdentity( offering.getId() );
                 
				Version offeringVersion = offering.getVersion();
				if(offeringVersion == null) offeringVersion = versionUnspecified;

				VersionRange offeringTolerance = offering.getTolerance();
				if(offeringTolerance == null) offeringTolerance = toleranceUnspecified;				
				
				IOffering offer = getUnspecifiedUtils( targetRepo ).findOffering(offeringId, offeringVersion, offeringTolerance, monitor);
				
				if(offer == null) throw new BuildException("Cannot locate offering: [" + offering.getId() + ", " + offeringVersion + "]");
							
				log("Offering: [" + offer.getIdentity().getId() + ", " + offer.getVersion() + "]");//$NON-NLS-1$
				List logs = transform(offer, targetRepo, monitor);
				print( logs );				
			}
            
		} catch (Exception e) {
			e.printStackTrace();
			throw new BuildException( e );
		}          
        finally {
            resetRepositoryGroup();
        }		
    }	
	
	protected abstract List transform(IOffering offering, IRepository repo, IProgressMonitor monitor) throws Exception;	
}
