package com.ibm.edp.cic.core.ant;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.service.resolver.VersionRange;

import com.ibm.cic.common.core.model.IAssembly;
import com.ibm.cic.common.core.model.IFix;
import com.ibm.cic.common.core.model.IIncludedShareableEntity;
import com.ibm.cic.common.core.model.IShareableEntity;
import com.ibm.cic.common.core.repository.IRepository;

public class CicReduceFix extends AbstractCicTransformFixMetadata 
{	
	protected List transform(IFix fix, IRepository repo, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
		
		IAssembly assembly = fix.getAssembly();
		if(assembly == null) throw new Exception( "Cannot locate assembly for [" + fix.getIdentity().getId() + "]" );
		
		//System.out.println( "SIZE: " + getRepositoryGroup().getAllShareableEntities(monitor).size() );
		checkAssembly(assembly, rc,  monitor);
		
		
		repo.deleteContent( fix );
		repo.refresh();
		repo.addContent( fix );
		
		return rc;
	}
	
    protected void checkAssembly(IAssembly assembly, List list, IProgressMonitor monitor)
    {
		for (Iterator iter = assembly.getChildren().iterator(); iter.hasNext();) {
			Object content = iter.next();
			if(content instanceof IIncludedShareableEntity)
			{
				IIncludedShareableEntity ise = (IIncludedShareableEntity)content;
				IShareableEntity se = ise.getShareableEntity();
				
				VersionRange tolerance = ise.getTolerance();
				
				if(se != null)
				{
				   //System.out.println( se.getIdentity() );				   
				   IShareableEntity availSE = getUnspecifiedUtils().findShareableEntity( ise.getIdentity(), versionUnspecified, toleranceUnspecified, monitor);
				   
				   //System.err.println( ise.getIdentity() + " - " + availSE );
				   
				   if(availSE == null)
				   {
					   list.add( "SE " + ise.getIdentity() + " is not available, so removing" );
					   iter.remove();
				   } 
				   else 
				   {

					   //Warn if cannot tolerate
					   if( !tolerance.isIncluded( availSE.getVersion() ) )
					   {
						   list.add( "[WARNING] SE " + ise.getIdentity() + " tolerance " +  tolerance + " does not tolerate version " + availSE.getVersion() + ", so removing" );
						   //iter.remove();					   
					   }					   
					   
					   list.add( "SE " + ise.getIdentity() + " is available, keeping" ); 
					   
					   reduceIncludedSE( ise, false );
				   }
				   
				}
				else
				{
				   list.add(  "SE " + ise.getIdentity() + " is null, so removing" );
				   iter.remove();
				}
			}
			else
			{
				list.add( "TBD: " + content.getClass().getName());	
			}			
		}
    	
    }
}
