package com.ibm.edp.cic.core.helper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.osgi.framework.Version;

import com.ibm.cic.common.core.model.CicFactory;
import com.ibm.cic.common.core.model.IAssembly;
import com.ibm.cic.common.core.model.IContentSelector;
import com.ibm.cic.common.core.model.IIdentity;
import com.ibm.cic.common.core.model.IIncludedShareableEntity;
import com.ibm.cic.common.core.model.IInstallableUnit;
import com.ibm.cic.common.core.model.IOffering;
import com.ibm.cic.common.core.model.ISelectionExpressionFactory;
import com.ibm.cic.common.core.model.IShareableEntity;
import com.ibm.cic.common.core.model.IncludedShareableEntitySelector;
import com.ibm.cic.common.core.model.Information;
import com.ibm.cic.common.core.model.SimpleIdentity;
import com.ibm.cic.common.core.model.adapterdata.IArtifact;
import com.ibm.cic.common.core.repository.IContentRepository;
import com.ibm.cic.common.core.repository.IRepository;
import com.ibm.cic.common.core.repository.RepositoryUtils;
import com.ibm.cic.common.downloads.IMutableContentInfo;

public class CicHelper {

	public static List exportShearableEntities(IOffering offering, IRepository targetRepo, boolean commit, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
		RepositoryUtils.resolve(offering, monitor);
		
		IRepository contentRepo = RepositoryUtils.getContentRepository(offering);
		List ses = contentRepo.getAllShareableEntities(monitor);
        for (Iterator iter = ses.iterator(); iter.hasNext();) 
        {
            IShareableEntity se = (IShareableEntity) iter.next();
            rc.add("Export shearableEntity [" + se.getIdentity().getId() + ", " + se.getVersion() + "]");
            if(commit) targetRepo.addContent( se );
        }
        return rc; 
	}	
	
	public static List resetBuildOfferingCheckSums(IOffering offering, boolean commit, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
		RepositoryUtils.resolve(offering, monitor);
		
		IContentRepository contentRepo = RepositoryUtils.getContentRepository(offering);

		/*
		    Do not need to overwrite repository, it is unlikely that there is a digest.
		    This operation removes NLS files
		List res1 = resetBuildOfferingCheckSums(offering.getInstallableUnits(), monitor);
    	rc.addAll( res1 );		
		
    	if(commit && rc.size() > 0)
    	{
	    	IRepository repo = offering.getRepository();
	    	
	    	repo.deleteContent( offering );	    	
	    	repo.addContent(offering);
    	}
    	*/
		
		List res2 = resetBuildOfferingCheckSums(contentRepo.getAllShareableEntities(monitor), contentRepo,  commit, monitor);        	
    	rc.addAll( res2 );        		
                
        return rc;
	}

	public static List resetBuildOfferingCheckSums(List seList, IRepository repo, boolean commit, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
		
		if(seList == null) return rc;
		
        for (Iterator se_iter = seList.iterator(); se_iter.hasNext();) 
        {
        	Object obj = se_iter.next();
        	IShareableEntity se = null;
        	/*
        	 
        	 Do not need to resolve included shareable entities because the code pick supp all SE 
        	 
        	if(obj instanceof IIncludedShareableEntity)
        	{
        		IIncludedShareableEntity ise = (IIncludedShareableEntity)obj;
        		se = findShareableEntity(ise, repo, monitor);
        	}
        	else if(obj instanceof IAssembly)
        	{               
        		IAssembly ass = (IAssembly)obj;
        	 	List res = resetBuildOfferingSE(ass.getChildren(), repo, monitor);        	
            	rc.addAll( res );    
            	se = ass;            	       
        	}
        	else
        	{
        		se = (IShareableEntity)obj;
        	}
        	*/        	
        	
        	se = (IShareableEntity)obj;
        	
        	if(se != null)
        	{       
        		//System.out.println("SE: " + se.getIdentity().getId());
	        	List iuList = CicUtil.collectIUs(se, monitor);	        	
	        	List res = resetBuildOfferingCheckSums(iuList, monitor);    
	        	if(commit && res.size() > 0)
	        	{
		        	repo.deleteContent( se );
		        	repo.addContent( se );
	        	}
	        	
	        	rc.addAll( res );
        	}
        }//for        	
        return rc;
	}    
    
	public static List resetBuildOfferingCheckSums(List iuList, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
        for (Iterator iu_iter = iuList.iterator(); iu_iter.hasNext();) 
        {
			IInstallableUnit iu = (IInstallableUnit) iu_iter.next();
	        for (Iterator iter = iu.getAdapterData().getArtifacts().iterator(); iter.hasNext();) 
	        {
	            IArtifact artifact = (IArtifact) iter.next();
	        	IMutableContentInfo info = artifact.getMutableContentInfo();
	        		        	
	        	//ContentInfoUtil.removeDigests(info);
	        		        	
	        	String [] digests = info.getAvailableMessageDigestAlgorithms();
	        	
	        	for (int i = 0; i < digests.length; i++) 
	        	{
	        		rc.add( "Digest " + digests[i] + " has been detected for artifact [" + artifact.getKey().getId() + ", " + artifact.getKey().getVersion() + "]");
	        		info.removeDigest( digests[i] );
				}//for    
	        	
	        }//for
        }//for        	
        return rc;
	}

    public static IShareableEntity findShareableEntity(IIncludedShareableEntity ise, IRepository repo, IProgressMonitor monitor) 
    {
        IShareableEntity se = ise.getShareableEntity();
        if (se == null) 
        {
    	   se = repo.findShareableEntity(ise.getIdentity(), ise.getVersion(), monitor);
    		
    		if(se == null)
    		{
    			System.err.println("Cannot find included SE [" + ise.getIdentity().getId() + ", " + ise.getVersion() + "]" );        			
    		}

        }
        return se;
    }
	
	public static void createBuildOffering(IRepository srcRepo, IRepository targetRepo, IIdentity offeringId, Version version, IProgressMonitor monitor) throws Exception
	{
		IIdentity assemblyId = getIdentity( offeringId.getId() + "_main" );
		
		IAssembly asm = createAssembly(assemblyId, version );
		
		IOffering ofr = createOffering(offeringId, assemblyId, version);
		ofr.setAssembly(asm);
		
		List sus = srcRepo.getAllShareableEntities( monitor );
		populateAssemblyWithSus(asm, sus);
		
		targetRepo.addContent(ofr);
	}
	
	public static IOffering createOffering(IIdentity offeringId, IIdentity assemblyId, Version version) 
	{
		IOffering offering = CicFactory.getInstance().createOffering(offeringId, version);
        offering.setAssemblyId( assemblyId );
        offering.setAssemblyVersion(version);
		offering.setInformation( getInformation(offeringId) );
		return offering;
	}
	
	public static IAssembly createAssembly( IIdentity assemblyId, Version version ) throws Exception {
		IAssembly assembly = CicFactory.getInstance().createAssembly(assemblyId, version);
		assembly.setInformation(getInformation(assemblyId));
		return assembly;
	}
	
    public static void populateAssemblyWithSus(IAssembly assembly, Collection allSus) {
        
    	CicFactory factory = CicFactory.getInstance();
    	
    	IIdentity main = new SimpleIdentity( assembly.getName() + "_selector" ); 
    	
    	assembly.addSelector( factory.createShareableEntitySelector( main ) );
    	
        for (Iterator iter = allSus.iterator(); iter.hasNext();) 
        {
            IShareableEntity se = (IShareableEntity) iter.next();
            
            IIncludedShareableEntity ise = factory.createIncludedShareableEntity(se);
            
            Set selectors = se.getSelectors();
            for (Iterator iterator = selectors.iterator(); iterator.hasNext();) {
            	
            	IContentSelector  selector = (IContentSelector) iterator.next();
            	
            	//if(selector instanceof InstallableUnitSelector)
            	{
	            	//InstallableUnitSelector iu_selector = (InstallableUnitSelector) selector;
	            	
	            	IncludedShareableEntitySelector ises = new IncludedShareableEntitySelector( selector.getIdentity() );
	            	ises.setExpression( ISelectionExpressionFactory.INSTANCE.createSelectedBy( main.getId() ) );
	            	ise.addIncludedSelector(ises);
            	}
			}
            
            /*            
            IncludedShareableEntitySelector ises = new IncludedShareableEntitySelector( main );
            ises.setExpression( ISelectionExpressionFactory.INSTANCE.createSelectedBy("main") );
            ise.addIncludedSelector(ises);
            
            
            IncludedShareableEntitySelector ises2 = new IncludedShareableEntitySelector( source );
            ises.setExpression( ISelectionExpressionFactory.INSTANCE.createSelectedBy("main") );
            ise.addIncludedSelector(ises2);
            */
            
            
            assembly.addChild(ise);
        }
    }
	
    public static IIdentity getIdentity( String strId) {
        return new SimpleIdentity(strId);
    }
	
	
	public static Information getInformation(IIdentity id) {
		Information info = new Information();
		info.setName(id.toString());
		return info;
	}		
}
